import { doc } from 'prettier';
import { NonterminalKind } from '@nomicfoundation/slang/cst';
import { printSeparatedList } from '../slang-printers/print-separated-list.js';
import { getNodeMetadata, updateMetadata } from '../slang-utils/metadata.js';
import { NamedArgument } from './NamedArgument.js';
const { line, softline } = doc.builders;
export class NamedArguments {
    constructor(ast, options) {
        this.kind = NonterminalKind.NamedArguments;
        let metadata = getNodeMetadata(ast, true);
        this.items = ast.items.map((item) => new NamedArgument(item, options));
        this.separators = ast.separators.map((separator) => separator.unparse());
        metadata = updateMetadata(metadata, [this.items]);
        this.comments = metadata.comments;
        this.loc = metadata.loc;
    }
    print(path, print, options) {
        return printSeparatedList(path.map(print, 'items'), {
            firstSeparator: options.bracketSpacing ? line : softline
        });
    }
}
//# sourceMappingURL=NamedArguments.js.map